Package: rules
Title: Model Wrappers for Rule-Based Models
Version: 1.0.2
Authors@R: c(
    person("Emil", "Hvitfeldt", , "emil.hvitfeldt@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0679-1945")),
    person("Max", "Kuhn", , "max@posit.co", role = c("aut"),
           comment = c(ORCID = "0000-0003-2402-136X")),
    person(given = "Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Bindings for additional models for use with the 'parsnip'
    package.  Models include prediction rule ensembles (Friedman and
    Popescu, 2008) <doi:10.1214/07-AOAS148>, C5.0 rules (Quinlan, 1992
    ISBN: 1558602380), and Cubist (Kuhn and Johnson, 2013)
    <doi:10.1007/978-1-4614-6849-3>.
License: MIT + file LICENSE
URL: https://github.com/tidymodels/rules, https://rules.tidymodels.org/
BugReports: https://github.com/tidymodels/rules/issues
Depends: parsnip (>= 0.2.1.9004), R (>= 3.4)
Imports: dials (>= 0.1.1.9001), dplyr, generics (>= 0.1.0), purrr,
        rlang, stats, stringr, tidyr, utils
Suggests: C50, covr, Cubist, knitr, modeldata, recipes, rmarkdown,
        spelling, testthat (>= 3.0.0), tibble, xrf (>= 0.2.0)
Config/Needs/website: tidyr, tidyverse/tidytemplate, recipes, xrf
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
Author: Emil Hvitfeldt [aut, cre] (<https://orcid.org/0000-0002-0679-1945>),
  Max Kuhn [aut] (<https://orcid.org/0000-0003-2402-136X>),
  Posit Software, PBC [cph, fnd]
Maintainer: Emil Hvitfeldt <emil.hvitfeldt@posit.co>
Built: R 4.4.0; ; 2024-05-08 17:08:54 UTC; unix
