# Generated by roxygen2: do not edit by hand

S3method(multi_predict,"_cubist")
S3method(multi_predict,"_xrf")
S3method(tidy,C5.0)
S3method(tidy,cubist)
S3method(tidy,xrf)
S3method(tunable,C5_rules)
S3method(tunable,cubist_rules)
S3method(tunable,rule_fit)
export(c5_fit)
export(c5_pred)
export(committees)
export(cubist_fit)
export(get_neighbors)
export(max_rules)
export(mtry_prop)
export(multi_predict)
export(tidy)
export(tunable)
export(xrf_fit)
export(xrf_pred)
importFrom(dials,mtry_prop)
importFrom(dials,new_quant_param)
importFrom(dplyr,"%>%")
importFrom(dplyr,as_tibble)
importFrom(dplyr,tibble)
importFrom(generics,tidy)
importFrom(generics,tunable)
importFrom(parsnip,multi_predict)
importFrom(purrr,map_dfr)
importFrom(rlang,abort)
importFrom(rlang,call2)
importFrom(rlang,enquos)
importFrom(rlang,eval_tidy)
importFrom(rlang,expr)
importFrom(rlang,warn)
importFrom(stats,model.frame)
importFrom(stats,model.response)
importFrom(stats,predict)
importFrom(stats,setNames)
importFrom(utils,head)
